'''Created Dec 3, 2021 Levi'''
import os
import platform
import subprocess as sp
import time
import traceback
import sys
from json import JSONDecodeError
from urllib.parse import urljoin
from collections import OrderedDict

import requests
import xlsxwriter as xlsw
from urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter

from .writer import Writer

def insert_keys(dict_, obj, pos):
    return {k: v for k, v in (list(dict_.items())[:pos] + list(obj.items()) + list(dict_.items())[pos:])}

def pretty_exc():
    exc_type, _exc_obj, exc_tb = sys.exc_info()
    # fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
    for tb in list(traceback.format_exception(exc_type, _exc_obj, exc_tb)):
        print(tb)

class anafAPI(object):
    '''classdocs
    '''
    BASEURL = 'https://webservicesp.anaf.ro/'
    VATURL = 'PlatitorTvaRest/api/v8/ws/tva'
    REPURL = 'bilant?an={}&cui={}'
    CULTURL = 'RegCult/api/v2/ws/cult'
    AGRURL = 'RegAgric/api/v2/ws/agric'

    def __init__(self):
        '''Constructor
        '''
        self.session = requests.Session()
        retry = Retry(connect=3, backoff_factor=1)
        adapter = HTTPAdapter(max_retries=retry)
        self.session.mount('http://', adapter)
        self.session.mount('https://', adapter)

    def parse_list(self, file_name=None, list_=None, excl_=[], ):
        if file_name:
            with open(file_name, 'r') as fiscal_codes, open("lista_cf_new.txt", "w") as fileout:
                for code in fiscal_codes:
                    if code[0:2] != 'HU' and code[0:2] != 'DE' and code[0:2] != 'GB' and code[0:2] != 'EL' and code[0:2] != 'LT' and 'SK' and code not in excl_:
                        fileout.write(code)
            with open("lista_cf_new.txt", "r") as newfile:
                self.lista_cf_ro = ' '.join(newfile).replace(',', '').split()
        if list_:
            lista_cf_ro = list_
        print('items in list', len(lista_cf_ro))
        return self.lista_cf_ro

    def get_vat(self, cui):
        url = urljoin(self.BASEURL, self.VATURL)
        payload = [{"cui": cui.replace('RO', ''), "data": time.strftime('%Y-%m-%d')}]
        result = self.session.post(url, json=payload, headers={'Content-Type': 'application/json'})
        try:
            resp_dict = result.json()['found'][0]
            # random_keys = ['telefon', 'fax', 'codPostal', 'act', 'stare_inregistrare']
            # pos = 5
            # for random_key in random_keys:
            #     if random_key not in resp_dict.keys():
            #         resp_dict = insert_keys(resp_dict, {random_key: 'Nan'}, pos)
            #     pos += 1
            processed_resp_dict = self.process_resp(resp_dict)
            new_dict = {processed_resp_dict['cui']: self._dict_to_list(processed_resp_dict, 0)}
        except (KeyError, JSONDecodeError, IndexError):
            print(result.text)
            pretty_exc()
            # pass
        time.sleep(1.1)
        return new_dict, processed_resp_dict
        # return resp_dict['date_generale']

    def get_cult(self, cui):
        url = urljoin(self.BASEURL, self.CULTURL)
        payload = [{"cui": cui.replace('RO', ''), "data": time.strftime('%Y-%m-%d')}]
        result = self.session.post(url, json=payload)
        try:
            resp_dict = result.json()['found'][0]
            new_dict = {resp_dict['cui']: self._dict_to_list(resp_dict, 0)}
        except (KeyError, JSONDecodeError):
            print(result.text)
            pretty_exc()
            # pass
        time.sleep(1.1)
        return new_dict, resp_dict

    def get_rep(self, cui, year):
        url = urljoin(self.BASEURL, self.REPURL.format(year, cui))
        result = self.session.get(url)
        print(result.status_code, result.text)
        resp_dict = result.json()
        # print(resp_dict)
        vals = [resp_dict[val] for val in resp_dict if val != 'i']
        indicators = resp_dict['i']
        for adict in indicators:
            # print(adict['indicator'])
            adict['indicator'] = int(adict['indicator'].replace('I', ''))
            # print(adict['indicator'])
        indicators = sorted(indicators, key=lambda d: d['indicator'])
        vals2 = [val["val_indicator"] for val in indicators]
        headers = [key for key in resp_dict if key != 'i']
        headers2 = ['{}-{}'.format(val['indicator'], val["val_den_indicator"]) for val in indicators]
        headers.extend(headers2)
        vals.extend(vals2)
        time.sleep(1.1)
        return headers, vals

    def to_excel(self, type_, cui=None, func=None, year=None, range_=None):
        types = {
            'to_rep': Writer(f'{type_}.xlsx').to_rep,
            'to_vat': Writer(f'{type_}.xlsx').to_vat,
            'to_cult': Writer(f'{type_}.xlsx').to_cult
        }
        types[type_](cui, func, year, range_)
        self.type_ = type_
        # if type_ == 'to_rep':
        #     Writer(f'{type_}.xlsx').to_rep(cui, func, year, range_)
        # elif type_ == 'to_vat':
        #     Writer(f'{type_}.xlsx').to_vat(cui, func)

    def open(self, file_name=None):
        file_name = file_name or f'{self.type_}.xlsx'
        abs_path = os.path.abspath(file_name)
        if platform.system() == 'Windows':
            sp.run(f'start {abs_path}', shell=True)
        elif platform.system() == 'Linux':
            sp.run(['xdg-open', f'{abs_path}'])
        else:
            raise NotImplementedError

    def _dict_to_list(self, s_dict, from_ind=None):
        if from_ind is None:
            n_list = [s_dict[i] for i in s_dict]
        else:
            n_list = [s_dict[i] for i in s_dict]
            n_list.pop(from_ind)
        return n_list

    def process_resp(self, response=None):
        # response = {'date_generale': {'cui': 28298293, 'data': '2023-05-25', 'denumire': 'BRIGHT CARS S.R.L.', 'adresa': 'JUD. ILFOV, SAT VOLUNTARI ORŞ. VOLUNTARI, STR. DRUMUL POTCOAVEI, NR.59 IX B', 'nrRegCom': 'J23/898/2011', 'telefon': '0753696891', 'fax': '', 'codPostal': '77190', 'act': '', 'stare_inregistrare': 'INREGISTRAT din data 04.04.2011', 'data_inregistrare': '2011-04-05', 'cod_CAEN': '4520', 'iban': '', 'statusRO_e_Factura': False, 'organFiscalCompetent': 'Administraţia Judeţeană a Finanţelor Publice Ilfov', 'forma_de_proprietate': 'PROPR.PRIVATA-CAPITAL PRIVAT AUTOHTON', 'forma_organizare': 'PERSOANA JURIDICA', 'forma_juridica': 'SOCIETATE COMERCIALĂ CU RĂSPUNDERE LIMITATĂ'},
        #         'inregistrare_scop_Tva': {'scpTVA': True, 'perioade_TVA': [{'data_inceput_ScpTVA': '2011-04-06', 'data_sfarsit_ScpTVA': '', 'data_anul_imp_ScpTVA': '', 'mesaj_ScpTVA': ''}]},
        #         'inregistrare_RTVAI': {'dataInceputTvaInc': '2013-01-01', 'dataSfarsitTvaInc': '2014-02-01', 'dataActualizareTvaInc': '2014-01-23', 'dataPublicareTvaInc': '2014-01-24', 'tipActTvaInc': 'Radiere', 'statusTvaIncasare': False},
        #         'stare_inactiv': {'dataInactivare': '', 'dataReactivare': '', 'dataPublicare': '', 'dataRadiere': '', 'statusInactivi': False},
        #         'inregistrare_SplitTVA': {'dataInceputSplitTVA': '', 'dataAnulareSplitTVA': '', 'statusSplitTVA': False},
        #         'adresa_sediu_social': {'sdenumire_Strada': 'Str. Drumul Potcoavei', 'snumar_Strada': '59 IX B', 'sdenumire_Localitate': 'Sat Voluntari Orş. Voluntari', 'scod_Localitate': '131', 'sdenumire_Judet': 'ILFOV', 'scod_Judet': '23', 'scod_JudetAuto': 'IF', 'stara': '', 'sdetalii_Adresa': '', 'scod_Postal': '77190'},
        #         'adresa_domiciliu_fiscal': {'ddenumire_Strada': 'Str. Drumul Potcoavei', 'dnumar_Strada': '59 IX B', 'ddenumire_Localitate': 'Sat Voluntari Orş. Voluntari', 'dcod_Localitate': '131', 'ddenumire_Judet': 'ILFOV', 'dcod_Judet': '23', 'dcod_JudetAuto': 'IF', 'dtara': '', 'ddetalii_Adresa': '', 'dcod_Postal': '77190'}}
        response.pop('inregistrare_SplitTVA')
        date = response['date_generale']
        tva = response['inregistrare_scop_Tva']
        tvai = response['inregistrare_RTVAI']
        stare = response['stare_inactiv']
        try:
            per_tva = tva['perioade_TVA'][0]
        except IndexError:
            per_tva = None
        tva.pop('perioade_TVA')
        if per_tva:
            all_data = OrderedDict(list(date.items()) + list(tva.items()) + list(per_tva.items()) + list(tvai.items()) + list(stare.items()))
        else:
            all_data = OrderedDict(list(date.items()) + list(tva.items()) + list(tvai.items()) + list(stare.items()))
        # all_data = self._dict_to_list(all_data, from_ind=0)
        # all_data = {**date, **tva, **tvai, **per_tva, **stare}
        return all_data
