'''Created 5 Jul 2022 Levi'''
import os
import ujson
import datetime
import io
import re
import random
from zipfile import ZipFile

from plyer import notification
from selenium import webdriver
from bs4 import BeautifulSoup
from requests.auth import HTTPBasicAuth
from requests_oauthlib import OAuth2Session
from requests.exceptions import JSONDecodeError

import anafapi.nextup
import ciel
from anafapi.en16931 import Invoice
from anafapi.nextup import client, TEMPLATES, MEAS_UNITS
# import ciel

os.environ['OAUTHLIB_INSECURE_TRANSPORT'] = '1'

class Anafoauth(object):

    AUTH_URL = 'https://logincert.anaf.ro/anaf-oauth2/v1/authorize'
    TOKEN_URL = 'https://logincert.anaf.ro/anaf-oauth2/v1/token'
    AUTH_URL_PARAMS = {'token_content_type': 'jwt'}
    TOKEN_URL_BODY = {'token_content_type': 'jwt'}

    def __init__(self, client_id=None, client_secret=None, redirect_uri=None, auth=None):
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.auth = auth or HTTPBasicAuth(self.client_id, self.client_secret)
        self.oauth = OAuth2Session(self.client_id, redirect_uri=self.redirect_uri)

    def load_token_file(self, path) -> dict:
        with open(path) as token_file:
            self.token_dict = ujson.load(token_file)
        return self.token_dict

    @property
    def is_valid(self) -> bool:
        default = 'unknow'
        expires_at = self.token_dict.get('expires_at', default)
        expires_at = datetime.datetime.fromtimestamp(expires_at)
        print(expires_at)
        return expires_at > datetime.datetime.today()

    def get_token(self) -> ujson:
        # https://chat.openai.com/c/61c5c1c5-298a-4073-88c7-d69ce99a140e
        # oauth = OAuth2Session(self.client_id, redirect_uri=self.redirect_uri)
        authorization_url, _state = self.oauth.authorization_url(self.AUTH_URL)
        #
        # for JWT token
        # authorization_url, state = self.oauth.authorization_url(self.AUTH_URL, params=AUTH_URL_PARAMS)
        #
        print(authorization_url)
        # self._browser_redirect(self.authorization_url)
        driver = webdriver.Chrome()
        driver.get(authorization_url)
        input('press enter after cert confirmation:')
        authorization_response = driver.current_url
        driver.close()
        start = authorization_response.index('code=') + 5
        end = authorization_response.index('&state=')
        auth_code = authorization_response[start:end]

        self.token = self.oauth.fetch_token(
            self.TOKEN_URL,
            auth=self.auth,
            code=auth_code,
            client_secret=self.client_secret)
        #
        # for JWT token we must add an additional body argument to fetch_token
        # body=TOKEN_URL_BODY
        #
        print(self.token)
        return self.token

    def revive_token(self) -> ujson:
        refr_token = self.token_dict['refresh_token']
        print(refr_token)
        self.token = self.oauth.refresh_token(
            token_url=self.TOKEN_URL,
            refresh_token=refr_token,
            auth=self.auth)
        print(self.token)
        return self.token

    def load_session(self):
        self.oauth = OAuth2Session(self.client_id, redirect_uri=self.redirect_uri, token=self.token_dict)

    def save_token(self, path) -> None:
        with open(path, 'w') as file:
            ujson.dump(self.token, file, indent=4, sort_keys=True)


class Efactoauth(Anafoauth):

    OAUTH = 'https://api.anaf.ro/TestOauth/jaxrs/hello?name="works with python"'
    MESSAGES = 'https://api.anaf.ro/{}/FCTEL/rest/listaMesajeFactura?zile={}&cif={}&filtru={}'
    LISTBYTIME = 'https://api.anaf.ro/{}/FCTEL/rest/listaMesajePaginatieFactura?startTime={}&endTime={}&cif={}&pagina={}'
    VERIFY = 'https://api.anaf.ro/{}/FCTEL/rest/stareMesaj?id_incarcare={}'
    ADD = 'https://api.anaf.ro/{}/FCTEL/rest/upload?standard={}&cif={}'
    DOWNLOAD = 'https://api.anaf.ro/{}/FCTEL/rest/descarcare?id={}'
    VALIDATE = 'https://webservicesp.anaf.ro/prod/FCTEL/rest/validare/{}'
    TRANSFORM = 'https://webservicesp.anaf.ro/prod/FCTEL/rest/transformare/{}/{}'

    def add_invoice(self, vat_id, fp, format_='UBL', env='test') -> dict:
        with open(fp, "r") as file:
            files = file.read()
            # headers = {'Content-Type': 'text/plain'}
            url = self.ADD.format(env, format_, vat_id)
            resp = self.oauth.post(url=url, data=files)
        try:
            print(resp.status_code, resp.json())
            return resp.json()
        except JSONDecodeError:
            print(resp.status_code, resp.text)
            return resp.text

    def validate_invoice(self, fp, type_='FACT1'):
        #
        # xmlns:schemaLocation="urn:oasis:names:specification:ubl:schema:xsd:Invoice-2../../UBL-2.1(1)/xsd/maindoc/UBL-Invoice-2.1.xsd"
        # must be deleted but why
        #
        with open(fp, "r") as file:
            files = file.read()
            files = files.replace('xmlns:schemaLocation="urn:oasis:names:specification:ubl:schema:xsd:Invoice-2../../UBL-2.1(1)/xsd/maindoc/UBL-Invoice-2.1.xsd" ', '')
            # print(files)
            headers = {'Content-Type': 'text/plain'}
            # response = requests.post(url, files=files)
            url = self.VALIDATE.format(type_)
            # print(url)
            resp = self.oauth.post(url=url, data=files.encode('utf-8'), headers=headers)
            print(resp.status_code, resp.text)
            # print(resp.status_code, resp.json())
            return resp.json()

    def transform_to_pdf(self, fp, type_='FACT1', no_validation=False):
        #
        # xmlns:schemaLocation="urn:oasis:names:specification:ubl:schema:xsd:Invoice-2../../UBL-2.1(1)/xsd/maindoc/UBL-Invoice-2.1.xsd"
        # must be deleted but why
        #
        with open(fp, "r") as file:
            files = file.read()
            files = files.replace(r'xmlns:schemaLocation="urn:oasis:names:specification:ubl:schema:xsd:Invoice-2../../UBL-2.1(1)/xsd/maindoc/UBL-Invoice-2.1.xsd" ', '')
            # print(files)
            headers = {'Content-Type': 'text/plain'}
            if no_validation:
                url = self.TRANSFORM.format(type_, no_validation)
            else:
                url = self.TRANSFORM.format(type_, '')
            resp = self.oauth.post(url=url, data=files, headers=headers)
            # print(resp.status_code, resp.json())
            with open(fp.replace('.xml', '.pdf'), 'wb') as output:
                output.write(resp.content)
            print(resp.status_code, resp.content)
            return resp.content

    def pretty_messages(self, vat_id, days=60, env='test') -> None:
        url = self.MESSAGES.format(env, days, vat_id)
        resp = self.oauth.get(url=url)
        resp_object = ujson.loads(resp.text)
        formatted_resp = ujson.dumps(resp_object, indent=4)
        print(resp.status_code, formatted_resp)

    def notify_new(self, date, cif, type_):
        notification.notify(title='E-INVOICE',
                            message=f'{type_} from {cif} @ {date}',
                            app_name='EFACT',
                            # app_icon=app_icon,
                            timeout=3)

    def get_all_messages(self, vat_id, days=60, env='test', filter_=None, root_path='efact', extract=False) -> dict:
        if filter_:
            url = self.MESSAGES.format(env, days, vat_id, filter_)
        else:
            self.MESSAGES = 'https://api.anaf.ro/{}/FCTEL/rest/listaMesajeFactura?zile={}&cif={}'
            url = self.MESSAGES.format(env, days, vat_id)
        resp = self.oauth.get(url=url)
        # print(resp.text)
        try:
            self.invoices = resp.json()
            # print(self.invoices)
            filtered_list = sorted([item for item in self.invoices['mesaje']], key=lambda x: datetime.date(int(x['data_creare'][:4]), int(x['data_creare'][4:6]), int(x['data_creare'][6:8])))
            self.invoices['mesaje'] = filtered_list
            # path = 'efact'
            for index_, inv in enumerate(self.invoices['mesaje'], start=1):
                date_ = datetime.date(int(inv['data_creare'][:4]), int(inv['data_creare'][4:6]), int(inv['data_creare'][6:8]))
                try:
                    cif = re.search('(?<=cif_emitent=)[0-9]+', inv['detalii']).group()
                    cif_ben = re.search('(?<=cif_beneficiar=)[0-9]+', inv['detalii']).group()
                except AttributeError:
                    cif = 'cif error'
                    cif_ben = 'cif_ben erro'
                type_ = inv['tip']
                print(index_, date_, cif, type_)
                # dir_dates = '{:02d}_{}'.format(date_.month, date_.year)
                dir_dates = '{}_{:02d}'.format(date_.year, date_.month)
                dir_cifs = '{}'.format(cif)
                dir_cifs_ben = '{}'.format(cif_ben)
                dir_types = type_
                dir_days = '{:02d}'.format(date_.day)
                if type_ == 'FACTURA TRIMISA':
                    # dirs = os.path.join(root_path, dir_cifs, dir_types, dir_dates, dir_days, dir_cifs_ben)
                    # dirs = os.path.join(root_path, dir_types, dir_dates, dir_cifs, dir_days, dir_cifs_ben)
                    dirs = os.path.join(root_path, dir_types, dir_dates, dir_days, dir_cifs, dir_cifs_ben)
                else:
                    # dirs = os.path.join(root_path, dir_cifs, dir_types, dir_dates, dir_days)
                    # dirs = os.path.join(root_path, dir_types, dir_dates, dir_cifs, dir_days)
                    dirs = os.path.join(root_path, dir_types, dir_dates, dir_days, dir_cifs)
                # print(dirs)
                os.makedirs(dirs, exist_ok=True)
                new = self.download_invoice(download_id=inv['id'], path=dirs, env=env)
                if new:
                    self.notify_new(date=date_, cif=cif, type_=type_)
                files = os.path.join(dirs, f'{inv["id"]}.zip')
                if extract:
                    self.getxml_from_zip(path=files, dest_f=dirs)
            return self.invoices
        except JSONDecodeError:
            soup = BeautifulSoup(resp.text, 'html.parser')
            print(resp.status_code, soup.get_text(strip=True))
            # print(resp.text, resp.status_code)
            return resp.text

    def download_invoice(self, download_id, path='efact', env='test') -> None:
        url = self.DOWNLOAD.format(env, download_id)
        resp = self.oauth.get(url=url)
        try:
            resp_text = ujson.loads(resp.text)
            formatted_resp = ujson.dumps(resp_text, indent=4)
            print(resp.status_code, formatted_resp)
            return False
        except ujson.JSONDecodeError:
            if not os.path.isfile(f'{path}/{download_id}.zip'):
                print('   downloading invoice in format zip')
                with open(f'{path}/{download_id}.zip', 'wb') as invoice_file:
                    invoice_file.write(resp.content)
                self.zip_inv = resp.content
                return True  # self.zip_inv
            # else:
            #     print('file already exists')
                # with open(f'{path}/{upload_id}.zip', 'r') as invoice_file:

    def verify_invoice(self, upload_id, env='test') -> str:
        url = self.VERIFY.format(env, upload_id)
        resp = self.oauth.get(url=url)
        # xml = bytes(bytearray(resp.text, encoding='utf-8'))
        # tree = etree.fromstring(xml)
        # formatted_tree = etree.tostring(tree, pretty_print=True)
        # print(formatted_tree)
        soup = BeautifulSoup(resp.text, 'xml')
        pretty_xml = soup.prettify()
        print(pretty_xml)
        print('stare', soup.header.get('stare'))
        print('id_descarcare', soup.header.get('id_descarcare'))
        return resp.text

    def _extract_zipfile(self, inv_zip, dest_f=None) -> os.path:
        for file_ in inv_zip.infolist():
            if not os.path.isfile(os.path.join(dest_f, file_.filename)):
                if 'semnatura' not in file_.filename:
                    _inv_file = inv_zip.extract(file_.filename, path=dest_f)
                    xml_inv = file_.filename
            elif 'semnatura' not in file_.filename:
                # print('file already extracted')
                xml_inv = os.path.join(dest_f, file_.filename)
        return xml_inv

    def getxml_from_zip(self, path=None, dest_f=None) -> os.path:
        if path:
            inv_zip = ZipFile(path, mode='r')
            name = [name for name in inv_zip.namelist() if 'semnatura' not in name]
            files_in_folder = os.listdir(dest_f)
            xml_files = [file for file in files_in_folder if file.lower().endswith('.xml')]
            # print(os.path.join(dest_f, name[0].replace('.xml', '') + '_booked.xml'))
            if not xml_files:
                if not os.path.exists(os.path.join(dest_f, name[0].replace('.xml', '') + '_booked.xml')):
                    self.xml_inv = self._extract_zipfile(inv_zip, dest_f=dest_f)
                # print(self.xml_inv)
                # if os.path.exists(self.xml_inv.replace('.xml', '') + '_booked.xml'):
                #     os.remove(self.xml_inv)
                    return self.xml_inv
        else:
            inv_zip = ZipFile(io.BytesIO(self.zip_inv), mode='r')
            name = [name for name in inv_zip.namelist() if 'semnatura' not in name]
            files_in_folder = os.listdir(dest_f)
            xml_files = [file for file in files_in_folder if file.lower().endswith('.xml')]
            if not xml_files:
                if not os.path.exists(os.path.join(dest_f, name[0].replace('.xml', '') + '_booked.xml')):
                    self.xml_inv = self._extract_zipfile(inv_zip, dest_f=dest_f)
                # print(self.xml_inv.replace('.xml', '') + '_booked.xml')
                # if os.path.exists(self.xml_inv.replace('.xml', '') + '_booked.xml'):
                #     os.remove(self.xml_inv)
                    return self.xml_inv

    def parse_inv(self, invoice: os.path=None) -> Invoice:
        if invoice:
            invoice = Invoice.from_xml(invoice)
        else:
            invoice = Invoice.from_xml(self.xml_inv)
        # print(invoice.seller_party.name, invoice.lines, str(int(invoice.lines[0].tax.percent * 100)), invoice.lines[0].price.amount)
        self.e_invoice = invoice
        return self.e_invoice

    def _parse_body(self, invoice: Invoice) -> str:
        art_list = [(line.quantity, line.item_name, line.price, line.currency) for line in invoice.lines]
        art_list = map(lambda x: str(x), art_list)
        art_list = "\n".join(art_list)
        body = f'''E-invoice:
        Nr. {invoice.invoice_id}
        date: {invoice.issue_date.strftime("%Y-%m-%d")}
        Issuer: {invoice.seller_party.name}
        Lines:\n{art_list}'''
        print(body)
        return body

    def _get_series(self, inv_id):
        series = re.search('[A-Z-]*', inv_id)
        if series:
            return series.group()
        else:
            series = 'EINV'
            return series

    def _get_number(self, inv_id):
        num = re.search('[0-9]+', inv_id)
        if num:
            return num.group()
        else:
            num = 1
            return num

    def addinvoice_tonextup(self, invoice: Invoice=None,
                            kind: str='stock',
                            template: int=32,
                            inc_plan: int=None,
                            exp_plan: int=None,
                            warehouse_code: int=23,
                            generic=False,
                            generic_value=None) -> None:
        '''accepted arguments for param
           @kind:
           stock
           service
           @templates:
           32 - consumables
           51 - obi
           30 - marfuri
           @generics:
           2209 - nestocate
           27 - servicii
           21 - intretinere
           22 - consumabile
           26 - cons auto
           488 - piese auto
           SER-CU - transport
           @planids:
           363 - 6022
           368 - 6028
           370 - 604
           383 - 622
           384 - 624
           691 - 6251
           389 - 628
           370 - 604
           440 - 704
           442 - 706
           444 - 708'''
        print('...getting articles')
        articles = client.getarticles_byname_andcode()
        names = [art['Name'].upper() for art in articles['Result']]
        print('...checking if invoice articles in nextup articles')
        lines = []
        series = self._get_series(inv_id=invoice.invoice_id)
        num = self._get_number(inv_id=invoice.invoice_id)
        if generic:
            for line in invoice.lines:
                line.item_code = generic_value
                lines.append(line)
        elif kind == 'stock':
            for line in invoice.lines:
                if line.item_name.upper() not in names:
                    code = random.randint(10000000, 99999999)
                    code = f'EN-{str(code)}'
                    line.item_code = code
                    print(f'...adding missing article <{line.item_code}>-<{line.item_name}>')
                    lines.append(line)
                    client.add_article(line.item_code,
                                       line.item_name.upper(),
                                       measure_unitid=MEAS_UNITS[line.unit_code],
                                       article_templateid=template)
                else:
                    codes = [article['Code'] for article in articles['Result'] if line.item_name.upper() == article['Name'].upper()]
                    line.item_code = ''.join(codes[0])
                    lines.append(line)
        elif kind == 'service':
            for line in invoice.lines:
                if line.item_name.upper() not in names:
                    code = random.randint(10000000, 99999999)
                    code = f'EN-{str(code)}'
                    line.item_code = code
                    print(f'...adding missing article <{line.item_code}>-<{line.item_name}>')
                    lines.append(line)
                    client.add_article(line.item_code,
                                       line.item_name.upper(),
                                       measure_unitid=MEAS_UNITS[line.unit_code],
                                       article_templateid=None,
                                       accounting_planid_incomeservice=inc_plan,
                                       accounting_planid_expenseservice=exp_plan,
                                       article_typeid=18)
                else:
                    codes = [article['Code'] for article in articles['Result'] if line.item_name.upper() == article['Name'].upper()]
                    line.item_code = ''.join(codes[0])
                    lines.append(line)
        invoice.all_lines = lines
        document_lines = client._parse_doc_lines(invoice.lines)
        post = client.add_invoice2(ciel.partners.PARTNERS[invoice.seller_party.name],
                                   lines=document_lines,
                                   date=invoice.issue_date.strftime('%Y-%m-%d'),
                                   warehouse_code=warehouse_code,
                                   nrdoc=num,
                                   series=series)
        if post['Error'] is None:
            # inv = ciel.Invoice(inv_series, inv_nr, inv_value, company_name, date_)
            env = anafapi.nextup.ciel.utils.mailer_details(anafapi.nextup.ciel.utils.TO, anafapi.nextup.ciel.utils.FR_ADDR)
            env._subject = f'E-invoice {invoice.invoice_id}/{invoice.issue_date.strftime("%Y-%m-%d")}-{invoice.seller_party.name} was booked'
            body = self._parse_body(invoice)
            env.add_to_text_body(body)
            print('...sending email notification')
            env.send(anafapi.nextup.ciel.utils.S_NAME, anafapi.nextup.ciel.utils.P_NUM, anafapi.nextup.ciel.utils.LOG_U, anafapi.nextup.ciel.utils.PAS, anafapi.nextup.ciel.utils.SEC)
        else:
            print(post['Error'])

    def get_latest_invoice(self, path):
        list_of_files = []
        for root, _, files in os.walk(path):
            for file in files:
                if file.lower().endswith('.xml'):
                    list_of_files.append(os.path.join(root, file))
        latest_file = max(list_of_files, key=os.path.getctime)
        # print(latest_file)
        self._latest_invoice = latest_file
        return self._latest_invoice


# if __name__ == '__main__':
#     s = 'https://www.anaf.ro/oauth2callback?code=d4dc641442a49fea4a8034da72a7d6b2f4360a0735b91ff014224d3363c0adf3&state=KjjDEPpKRINqZzzrdP5xGCD6ZQViKO'
#     # print(s.index('code='))
#     start = s.index('code=') + 5
#     end = s.index('&state=')
#
#     print(s[start:end])
